Add-Type -AssemblyName PresentationCore, PresentationFramework

$parentDir = $PSScriptRoot + "\.."
$levelDir = $parentDir + "\.."

Function Get-Folder($initialDirectory = $parentDir) {
  [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null

  $foldername = New-Object System.Windows.Forms.FolderBrowserDialog
  $foldername.Description = "Select your GameData folder (containing jasp.exe, jamp.exe, base folder, MBII folder, etc.)"
  $foldername.rootfolder = "MyComputer"
  $foldername.SelectedPath = $initialDirectory

  if ($foldername.ShowDialog() -eq "OK") {
    $folder += $foldername.SelectedPath
  }
  return $folder
}

Function Test-GameData($gameData) {
  if ($null -eq $gameData) {
    return $false
  }
  elseif (-Not ((Test-Path $gameData\jamp.exe) -AND (Test-Path $gameData\base\assets3.pk3))) {
    return $false
  }
  return $true
}

Function Set-Shortcut($name, $exe, $arg, $work, $icon) {
  $WshShell = New-Object -comObject WScript.Shell
  $Shortcut = $WshShell.CreateShortcut("$parentDir\$name.lnk")
  $Shortcut.TargetPath = "$exe"
  if ($arg) {
    $Shortcut.Arguments = "$arg"
  }
  if ($work) {
    $Shortcut.WorkingDirectory = "$work"
  }
  if ($icon) {
    $Shortcut.IconLocation  = "$icon"
  }
  $Shortcut.Save()
}

Function Install-Radiants() {
  # $gameData = "C:\Projects\Coding\launcher\MBIILauncher_WPF\bin\GameData"
  $gameData = Get-Folder
  $validGameData = Test-GameData($gameData)
  while ($validGameData -ne $true) {
    $folderFailPrompt = [System.Windows.MessageBox]::Show("Invalid GameData path, try again?", "Map Editor Setup", "OKCancel", "Error")
    switch ($folderFailPrompt) {
      "OK" {
        $gameData = Get-Folder
      }
      "Cancel" {
        Write-Host "Stopping install script"
        Exit 0
      }
    }
    $validGameData = Test-GameData($gameData)
  }
  Write-Host "GameData Path:" $gameData

  ########################

  Write-Host "Copying editor assets to Base"
  Expand-Archive -Path "$levelDir\Assets\SDK.zip" -DestinationPath "$gameData\base" -Force
  Get-ChildItem "$levelDir\Assets\*.pk3" | Copy-Item -Destination "$gameData\base"

  ########################

  Write-Host "Configuring NetRadiant settings"
  $gameDataUnix = $gamedata -replace "\\", "/"
  $netradiantSettings = "$PSScriptRoot\netradiant-custom\settings\1.6.0\ja.game"
  $netradiantSettingsBase = "$PSScriptRoot\netradiant-custom\gamepacks\ja.game\base"
  if (Test-Path "$netradiantSettings\local.pref") {
    Copy-Item -Path "$netradiantSettings\local.pref" -Destination "$netradiantSettings\local_$(Get-Date -f yyyy-MM-dd_HH-mm-ss).pref"
    $raw = Get-Content -Path "$netradiantSettings\local.pref" -Raw
    $raw -ireplace '(<epair name="EnginePath">)[^<]+(</epair>)', "`$1$gameDataUnix/`$2" | Out-File -FilePath "$netradiantSettings\local.pref" -Encoding "utf8"
  }
  else {
    New-Item -Path $netradiantSettings\local.pref -ItemType "file" -Force -Value @"
<?xml version="1.0"?>
<qpref version="1.0">
<epair name="EnginePath">$gameDataUnix/</epair>
<epair name="GameMode">mp</epair>
<epair name="GameName">MBII</epair>
</qpref>
"@ | Out-Null
  }

  ########################

  Write-Host "Configuring GtkRadiant settings"
  $gtkradiantSettings = "$PSScriptRoot\gtkradiant-16\installs\JAPack\game"
  $gtkradiantSettingsBase = "$gameData\base\scripts"
  $gtkradiantSettingsGame = "$PSScriptRoot\gtkradiant-16\games"

  $raw = Get-Content -Path "$gtkradiantSettingsGame\default_ja.game" -Raw
  $raw = $raw -ireplace 'REPLACEMEWITHACTUALGAMEDATA', "$gameData"
  $raw -ireplace 'REPLACEMEWITHACTUALPROJ', "$gtkradiantSettings" | Out-File -FilePath "$gtkradiantSettingsGame\ja.game" -Encoding "utf8"
  
  Copy-Item -Path "$gtkradiantSettings\default_project.proj" -Destination "$gtkradiantSettingsBase"

  ########################

  Write-Host "Configuring Q3map2GUI"
  $q3map2guiSettings = "$PSScriptRoot\q3map2gui"
  Copy-Item -Path "$q3map2guiSettings\default_q3map2gui.ini" -Destination "$q3map2guiSettings\q3map2gui.ini"
  $raw = Get-Content -Path "$q3map2guiSettings\q3map2gui.ini" -Raw
  $raw = $raw -ireplace 'REPLACEMEWITHACTUALGAMEDATA', "$gameData"
  $raw = $raw -ireplace 'REPLACEMEWITHACTUALQ3MAP2', "$PSScriptRoot\gtkradiant-16\x64\q3map2.exe"
  $raw -ireplace 'REPLACEMEWITHACTUALLOGS', "$q3map2guiSettings\logs" | Out-File -FilePath "$q3map2guiSettings\q3map2gui.ini" -Encoding "utf8"

  ########################6

  Write-Host "Copying MBII entity definitions and default shaderlist"
#  Copy-Item -Path "$levelDir\Entities\Combined\mp_entities.def" -Destination "$netradiantSettingsBase"
  Copy-Item -Path "$levelDir\Entities\Combined\mp_entities.def" -Destination "$gtkradiantSettingsBase"
#  Copy-Item -Path "$levelDir\Assets\shaderlist.txt" -Destination "$netradiantSettingsBase\default_shaderlist.txt"
  Copy-Item -Path "$levelDir\Assets\shaderlist.txt" -Destination "$gtkradiantSettingsBase\default_shaderlist.txt"
  # Because 1.6 is ass and doesn't use default_shaderlist, copy if missing
  if (-Not (Test-Path "$gameData\base\shaders\shaderlist.txt")) {
    Copy-Item -Path "$levelDir\Assets\shaderlist.txt" -Destination "$gameData\base\shaders"
  }

  ########################

  Write-Host "Creating shortcuts"
  Set-Shortcut -name "NetRadiant" -exe "$PSScriptRoot\netradiant-custom\radiant.exe"
  Set-Shortcut -name "GtkRadiant" -exe "$PSScriptRoot\gtkradiant-16\radiant.exe"
  Set-Shortcut -name "Q3map2GUI" -exe "C:\Windows\System32\cmd.exe" -arg "/c start `"Q3map2GUI`" /belownormal `"$PSScriptRoot\q3map2gui\q3map2gui.exe`"" `
    -work "$PSScriptRoot\q3map2gui" -icon "$PSScriptRoot\q3map2gui\q3map2gui.ico"
}


$initPrompt = [System.Windows.MessageBox]::Show("Configure map editors?", "Map Editor Setup", "OKCancel", "Question")
switch ($initPrompt) {
  "OK" {
    Install-Radiants
  }
  "Cancel" {
    Exit 0
  }
}

